#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PIL import Image
import sys
from struct import pack

pl=[(0,0,0,25), (0,0,0), (62,184,73), (116,208,125),
  (89,85,224), (128,118,241), (185,94,81), (101,219,239),
  (219,101,89), (255,137,125), (204,195,94), (222,208,135),
  (58,162,65), (183,102,181), (204,204,204), (255,255,255)
]

def dist(c1,c2):
  if type(c1)==int: c1=pl[c1]
  if type(c2)==int: c2=pl[c2]
  r=c1[0]-c2[0]
  g=c1[1]-c2[1]
  b=c1[2]-c2[2]
  return(r*r+g*g+b*b)

def toCC(pix):
  return sorted([(dist(pix,c),i) for i,c in enumerate(pl)])[0][1]

def cnv(srcimg,dstsize,pcg=8,ncolors=2):
  img=Image.new('RGB',dstsize)
  img.paste(srcimg)
  ptn=[]
  col=[]
  for y in range(0,img.height,pcg):
    for x in range(0,img.width,pcg):
      for yy in range(y,y+pcg):
        ps=[toCC(img.getpixel((x+i,yy))) for i in range(pcg)]
        cs={i:0 for i in set(ps)}
        for p in ps: cs[p]+=1
        cs=sorted(cs,key=lambda i:cs[i],reverse=True)[:ncolors]
        if len(cs)<2: cs*=2
        p=0
        for i in range(pcg):
          c=sorted([(dist(ps[i],c),j) for j,c in enumerate(cs)])[0][1]
          img.putpixel((x+i,yy), pl[cs[c]])
          p+=c<<(7-i)
        ptn.append(p)
        col.append(cs[1]<<4|cs[0])
  return img,ptn,col

if __name__ == '__main__':
  for fn in sys.argv[1:]:
    img,ptn,col=cnv(Image.open(fn),(256,192))
    if img:
      img.save(fn+"sc2.png")
      with open(fn+"sc2.bin","wb") as f:
        f.write(pack("%dB"%(7), 0xfe,0,0,0x00,0x38,0,0))
        f.write(pack("%dB"%(len(ptn)), *ptn))
        f.write(pack("%dB"%(3*256), *[j for i in range(3) for j in range(256)]))
        f.write("\xd8\x00\x00\x00"*32)
        f.write("\x00"*0x480)
        f.write(pack("%dB"%(len(col)), *col))
        f.flush()

