;===============================================================
;
;	Super Laydock
;
;		割り込み処理
;
;===============================================================
;SPG	#1800	r6:3
;PNT	#3800	r2:0e
;SAT	#3c00	r5:78 r11:0

;PGT0	#0000	r4:03
;PGT1	#2000	r4:07
;PGT2	#4000	r4:0b
;PGT3	#6000	r4:0f
;PGT4	#8000	r4:13
;PGT5	#a000	r4:17
;PGT6	#c000	r4:1b
;PGT7	#e000	r4:1f

;CT0	#10000	r2:7f r3:4
;CT1	#12000	r2:ff r3:4
;CT2	#14000	r2:7f r3:5
;CT3	#16000	r2:ff r3:5
;CT4	#18000	r2:7f r3:6
;CT5	#1a000	r2:ff r3:6
;CT6	#1c000	r2:7f r3:7
;CT7	#1e000	r2:ff r3:7



;===============================================================
;
;	割り込みセット
;
;===============================================================
hkey_setup:
proc
		ld		hl,hkey_src
		ld		de,hkey_org
		ld		bc,end_hkey-my_hkey
		ldir

		xor		a
		ld		(msg_area),a
		ld		(disp_flag),a
		ld		(#e2ec),a
		ld		(#e42c),a
		dec		a
		ld		(cur_weapon),a
		ld		(cur_message),a

		; ステージ別に割り込みを分ける
		ld		a,(#e3ca)
		cp		3
		ld		hl,my_hkey_3
		jr		z,_set_hkey
		cp		6
		ld		hl,my_hkey_6
		jr		z,_set_hkey
		ld		hl,my_hkey_normal
	_set_hkey:
		ld		(#fd9f+1),hl

		ld		c,#99
		; PNTを#3800に
		ld		hl,#820e
		out		(c),l
		out		(c),h
		; SATを#3c00に
		ld		hl,#8578
		out		(c),l
		out		(c),h
		; SPGを#1800に
		ld		hl,#8603
		out		(c),l
		out		(c),h

		; スクロール追加処理
		call	is_stage_3_6
		ret		nz
	_set_next_bg:
		ld		hl,next_bg
		ld		de,next_bg+1
		ld		bc,23
		ld		(hl),#b8
		ldir
		ret
endp


;===============================================================
;
;	割り込み処理
;
;===============================================================
hkey_src:
org			hkey_org
my_hkey:
my_hkey_6:
proc
		di
		ld		hl,(#e379)
		ld		a,l
		or		h
		ld		a,(#e42c)
		jr		nz,_not_brake
		neg
	_not_brake:
		; 256周期を8周期に	0 1 2 3 4 5 6 7
		rlca
		rlca
		rlca
		and		7
		ld		e,a
		jp		my_hkey_cmn
endp
my_hkey_3:
proc
		di
		ld		a,(#e424)
		cp		4
		ld		a,(#e42c)
		jr		nz,_not_brake
		neg
	_not_brake:
		; 256周期を8周期に	0 1 2 3 4 5 6 7
		rlca
		rlca
		rlca
		and		7
		ld		e,a
		jp		my_hkey_cmn
endp
my_hkey_normal:
proc
		di
		; 24周期を8周期に	0 1 2 3 4 5 6 7
		ld		a,(#e2ec)
		ld		de,#03ff
	_lp_div3:
		inc		e
		sub		d
		jr		nc,_lp_div3
		ld		a,e
endp
my_hkey_cmn:
proc
		; フェードアウト中はスクロールさせない
		ld		a,(disp_flag)
		rlca
		jr		nc,_scroll_ok
		ld		a,(cur_page)
		ld		e,a
	_scroll_ok:
		ld		a,e
		ld		(cur_page),a

		; VDP i/oポート
		ld		c,#99	;もともと直値^^

		;--------------------------------
		; PGC,CT,Yオフセット
		;--------------------------------
		add		a,a
		add		a,a
		ld		hl,_vdp_params
		add		a,l
		ld		l,a
		jr		nc,_nc_vdp_params
		inc		h
	_nc_vdp_params:
		ld		a,#8a	;vdpレジスタ番号
		ld		b,#85	;vdpレジスタ番号
		outi
		out		(c),b	; 4
		outi
		out		(c),b	; 3
		outi
		out		(c),a	; 10

		; スクロール値		0 -1 -2 -3 -4 -5 -6 -7
		xor		a
		sub		e
		ld		h,#97
		out		(c),a
		out		(c),h

		; スプライトY補正値	0 -1 -2 -3 -4 -5 -6 -7
		ld		e,a

		; 当たり判定補正
		ld		(spr_bg_ofs),a

		;--------------------------------
		; vdp上位アドレス
		;--------------------------------
		ld		hl,#8e00
		out		(c),l
		out		(c),h

		;--------------------------------
		; ページ2でPGTとPNTが被る問題回避
		;--------------------------------
		ld		hl,#7bfa
		out		(c),l
		out		(c),h
		ld		a,(cur_page)
		dec		a
		ld		a,#17
		jr		nz,_tweak_not_page2
		ld		a,#e0
	_tweak_not_page2:
		dec		c
		out		(c),a
		add		a,8
		out		(c),a
		inc		c

		;--------------------------------
		; SAT転送
		;--------------------------------
		ld		hl,#7c00
		out		(c),l
		out		(c),h

		dec		c
		ld		d,#0c

		ld		a,(#e2e8)
		add		a,#1c
		and		#7c
		ld		(#e2e8),a
		add		a,d
		and		#7c

		ld		b,#80-#20
		ld		h,#e0
	_lp_sat:
		ld		l,a
		ld		a,(hl)
		add		a,e
		out		(c),a
		ld		a,l
		add		a,d
		inc		l
		outi
		and		#7c
		outi
		nop
		nop
		outi
		jp		nz,_lp_sat

		;--------------------------------
		; PNT転送
		;--------------------------------
		ld		a,(disp_flag)
		or		a
		jr		nz,_skip_pnt
		;PNT転送
		ld		hl,#e0a8
		ld		de,#7818
		xor		a
	_lp_pnt:
		inc		c
		out		(c),a
		out		(c),d
		dec		c
		ld		b,#18
		otir
		add		a,#20
		jr		nc,_next_pnt
		inc		d
	_next_pnt:
		dec		e
		jp		nz,_lp_pnt
		;PNT転送一行追加
		inc		c
		ld		hl,#7be0
		out		(c),l
		out		(c),h
		dec		c
		ld		hl,next_bg
		ld		b,#18
		otir
	_skip_pnt:

		;--------------------------------
		; メッセージ表示
		;--------------------------------
		ld		hl,msg_area
		ld		a,(hl)
		or		a
		jr		z,_end_msg
		add		a,2
		ld		b,a
		xor		a
		ld		(hl),a
		inc		hl
		inc		c
		outi
		outi
		dec		c
		otir
	_end_msg:

		;--------------------------------
		; 音関係
		;--------------------------------
		call	#4577      
		xor		a          
		ld		(#e2eb),a

		ei
		ret

		;--------------------------------
		; VDP設定
		;--------------------------------
	_vdp_params:
		db	#03,#7f,#04,0
		db	#07,#ff,#04,0
		db	#0b,#7f,#05,0
		db	#0f,#ff,#05,0
		db	#13,#7f,#06,0
		db	#17,#ff,#06,0
		db	#1b,#7f,#07,0
		db	#1f,#ff,#07,0
endp


copy_next_bg_on_init:
		call	is_stage_3_6
		ret		z
copy_next_bg:
		ld		hl,(#e2e9)
		ld		de,-24-1
		add		hl,de
		ld		de,next_bg
		ld		bc,24
		ldir
		ret
copy_next_bg_on_scroll:
		call	copy_next_bg
		jp		#5307

is_stage_3_6:
		ld		a,(#e3ca)
		cp		3
		ret		z
		cp		6
		ret

update_is_scroll:
proc
		ld		a,(#e3ca)
		ld		hl,_tbl
		cp		3
		jr		z,_3_6
		cp		6
		jr		nz,_set
	_3_6:
		dec		hl
		dec		hl
	_set:
		ld		a,(0xe2ec)
		add		a,l
		ld		l,a
		ld		a,(hl)
		ld		(is_scroll),a
		ret
	_tbl2:
		db		8,0
	_tbl:
		db		1,0,0,1,0,0,1,0,0,1,0,0
		db		1,0,0,1,0,0,1,0,0,1,0,0
endp


end_hkey:
org		hkey_src+end_hkey-hkey_org
;===============================================================
